@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -32,7 +32,10 @@
 -include("ssl_record.hrl").
 -include("ssl_internal.hrl").
 
--export([decode/1, own_alert_txt/1, alert_txt/1, reason_code/2]).
+-export([decode/1, 
+         own_alert_format/4, 
+         alert_format/4,
+         reason_code/4]).
 
 %%====================================================================
 %% Internal application API
@@ -48,54 +51,79 @@ decode(Bin) ->
     decode(Bin, [], 0).
 
 %%--------------------------------------------------------------------
-%% -spec reason_code(#alert{}, client | server) ->
-%%                          {tls_alert, unicode:chardata()} | closed.
-%-spec reason_code(#alert{}, client | server) -> closed | {essl, string()}.
+-spec reason_code(#alert{}, client | server, ProtocolName::string(), StateName::atom()) ->
+                         {tls_alert, {atom(), unicode:chardata()}} | closed.
 %%
 %% Description: Returns the error reason that will be returned to the
 %% user.
 %%--------------------------------------------------------------------
 
-reason_code(#alert{description = ?CLOSE_NOTIFY}, _) ->
+reason_code(#alert{description = ?CLOSE_NOTIFY}, _, _, _) ->
     closed;
-reason_code(#alert{description = Description, role = Role} = Alert, Role) ->
-    {tls_alert, {description_atom(Description), own_alert_txt(Alert)}};
-reason_code(#alert{description = Description} = Alert, Role) ->
-    {tls_alert, {description_atom(Description), alert_txt(Alert#alert{role = Role})}}.
+reason_code(#alert{description = Description, role = Role} = Alert, Role, ProtocolName, StateName) ->
+    {PrefixFmt, PrefixArgs} = alert_prefix_format(ProtocolName, Role, StateName),
+    {Fmt, Args} = own_alert_format_depth(Alert),
+    Txt = lists:flatten(io_lib:format(PrefixFmt ++ Fmt, PrefixArgs ++ Args)),
+    {tls_alert, {description_atom(Description), Txt}};
+reason_code(#alert{description = Description} = Alert, Role, ProtocolName, StateName) ->
+    {Fmt, Args} = alert_format(ProtocolName, Role, StateName, Alert),
+    Txt = lists:flatten(io_lib:format(Fmt, Args)),
+    {tls_alert, {description_atom(Description), Txt}}.
 
 %%--------------------------------------------------------------------
--spec own_alert_txt(#alert{}) -> string().
+-spec own_alert_format(string(), server | client, StateNam::atom(), #alert{}) -> {io:format(), list()}.
 %%
-%% Description: Returns the error string for given alert generated
-%% by the erlang implementation.
+%% Description: Generates alert text for log or string part of error return.
 %%--------------------------------------------------------------------
-own_alert_txt(#alert{level = Level, description = Description, where = {Mod,Line}, reason = undefined, role = Role}) ->
-    "at " ++ Mod ++ ":" ++ integer_to_list(Line) ++ " generated " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
-        level_txt(Level) ++ description_txt(Description);
-own_alert_txt(#alert{reason = Reason} = Alert) ->
-    BaseTxt = own_alert_txt(Alert#alert{reason = undefined}),
-    FormatDepth = 9, % Some limit on printed representation of an error
-    ReasonTxt = lists:flatten(io_lib:format("~P", [Reason, FormatDepth])),
-    BaseTxt ++ " - " ++ ReasonTxt.
+own_alert_format(ProtocolName, Role, StateName, Alert) ->
+    {PrfixFmt, PrefixArgs} = alert_prefix_format(ProtocolName, Role, StateName),
+    {Fmt, Args} = own_alert_format(Alert),
+    {PrfixFmt ++ Fmt, PrefixArgs ++ Args}.
 
 %%--------------------------------------------------------------------
--spec alert_txt(#alert{}) -> string().
+-spec alert_format(string(), server | client, StateNam::atom(), #alert{}) -> {io:format(), list()}.
 %%
-%% Description: Returns the error string for given alert received from
-%% the peer. 
+%% Description: Generates alert text for log or string part of error return.
 %%--------------------------------------------------------------------
-alert_txt(#alert{level = Level, description = Description, reason = undefined, role = Role}) ->
-    "received " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
-        level_txt(Level) ++ description_txt(Description);
-alert_txt(#alert{reason = Reason} = Alert) ->
-    BaseTxt = alert_txt(Alert#alert{reason = undefined}),
-    FormatDepth = 9, % Some limit on printed representation of an error
-    ReasonTxt = lists:flatten(io_lib:format("~P", [Reason, FormatDepth])),
-    BaseTxt ++ " - " ++ ReasonTxt.
+alert_format(ProtocolName, Role, StateName, Alert) ->
+    {PrfixFmt, PrefixArgs} = alert_prefix_format(ProtocolName, Role, StateName),
+    {Fmt, Args} = alert_format(Alert),
+    {PrfixFmt ++ Fmt, PrefixArgs ++ Args}.
 
 %%--------------------------------------------------------------------
 %%% Internal functions
 %%--------------------------------------------------------------------
+alert_prefix_format(ProtocolName, Role, StateName) ->
+    {"~s ~p: In state ~p", [ProtocolName, Role, StateName]}.
+
+own_alert_format(#alert{reason = Reason} = Alert) ->   
+    Txt = own_alert_txt(Alert),
+    case Reason of
+        undefined ->            
+            {" ~s\n", [Txt]};
+        Reason ->
+            {" ~s\n - ~p", [Txt, Reason]}
+    end.
+own_alert_format_depth(#alert{reason = Reason} = Alert) ->   
+    Txt = own_alert_txt(Alert),
+    case Reason of
+        undefined ->            
+            {" ~s\n", [Txt]};
+        Reason ->
+            {" ~s\n ~P", [Txt, Reason, ?DEPTH]}
+    end.
+
+own_alert_txt(#alert{level = Level, description = Description, where = #{line := Line, file := Mod}, role = Role}) ->
+    "at " ++ Mod ++ ":" ++ integer_to_list(Line) ++ " generated " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
+        level_txt(Level) ++ description_txt(Description).
+
+alert_format(Alert) ->
+    Txt = alert_txt(Alert),
+    {" ~s\n ", [Txt]}.
+
+alert_txt(#alert{level = Level, description = Description, role = Role}) ->
+    "received " ++ string:uppercase(atom_to_list(Role)) ++ " ALERT: " ++
+        level_txt(Level) ++ description_txt(Description).
 
 %% It is very unlikely that an correct implementation will send more than one alert at the time
 %% So it there is more than 10 warning alerts we consider it an error
@@ -173,8 +201,8 @@ description_txt(?UNSUPPORTED_EXTENSION) ->
     "Unsupported Extension";
 description_txt(?CERTIFICATE_UNOBTAINABLE) ->
     "Certificate Unobtainable";
-description_txt(?UNRECOGNISED_NAME) ->
-    "Unrecognised Name";
+description_txt(?UNRECOGNIZED_NAME) ->
+    "Unrecognized Name";
 description_txt(?BAD_CERTIFICATE_STATUS_RESPONSE) ->
     "Bad Certificate Status Response";
 description_txt(?BAD_CERTIFICATE_HASH_VALUE) ->
@@ -244,8 +272,8 @@ description_atom(?UNSUPPORTED_EXTENSION) ->
     unsupported_extension;
 description_atom(?CERTIFICATE_UNOBTAINABLE) ->
     certificate_unobtainable;
-description_atom(?UNRECOGNISED_NAME) ->
-    unrecognised_name;
+description_atom(?UNRECOGNIZED_NAME) ->
+    unrecognized_name;
 description_atom(?BAD_CERTIFICATE_STATUS_RESPONSE) ->
     bad_certificate_status_response;
 description_atom(?BAD_CERTIFICATE_HASH_VALUE) ->
